function lrn_par = Set_Default_Learning_Paramters

% function lrn_par = Set_Default_Learning_Paramters
%
% Returns the default values for lrn_par (to be used with LG_Cluster)
%
%
% Clustering Through Ranking On Manifolds
% Version 0.2
%
% Copyright by Markus Breitenbach and Gregory Z. Grudic
% This code is for your personal and research use only.
%
% http://www.cs.colorado.edu/~grudic/
% http://ucsu.colorado.edu/~breitenm/
%
% This software is provided "as is," without warranty of any kind, express
% or implied.  In no event shall the authors be held liable
% for any direct, indirect, incidental, special or consequential damages
% arising out of the use of or inability to use this software.
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% 
%%% Set the parameters
%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Experimental
lrn_par.preSet_Centers=[]; % an index to "find" a particular cluster center first
lrn_par.Cost_function=1;   % which cost-function for the estimation
lrn_par.SIGMA_MEDIAN = 0;
lrn_par.SIGMA_MEDIAN_K = 5;

% distance paramters
lrn_par.DST_TYPE = 1; % 1 for Euclidean, 2 for dot
lrn_par.my_alpha = 0; % if set to zero, algorithm with pick a value
lrn_par.my_sigma = 0; % if set to zero, algorithm will pick a value

% scaling data paramters
lrn_par.SCALE = 1; % scale data if set to 1;

% debug parameters
lrn_par.DST_SAVED = 0; % set to one if distance matrix is pre-calculated

% SIGMA optimization
lrn_par.SIGMA_MaxFunEvals = 200; % maximum number of function evaluations for sigma estimation
lrn_par.SIGMA_ALPHA_MaxFunEvals = 200; % maximum number of function evaluations for sigma/alpha estimation
lrn_par.OPT_SIGMA_MIN_INTERVEL = 0.0005; % maximum subsample of points for initial sigma estimation
lrn_par.OPT_SIGMA_MAX_INTERVEL = 1.0; % maximum subsample of points for initial sigma estimation
lrn_par.OPT_SIGMA_DEL_INTERVEL = 0.005; % maximum subsample of points for initial sigma estimation

% The default interval of the number of clusters
lrn_par.Cluster_Search = 2:5;

% Optimize for sigma+alpha at the same time
lrn_par.OPT_S_A=1;

% If windows is used, we can use the fast C-code...
lrn_par.USE_C_CODE_FOR_DST = 0;
path=pwd;
if length(find(path=='\'))>0
    % it seems that we have a windows-path...
    lrn_par.USE_C_CODE_FOR_DST = 1; % If this does not work for you, change this to 0.
    fprintf(1,'Detected Matlab on Windows - Using compiled distance function\n');
end;

