function batch_3_Spiral_3D

%
% Clustering Through Ranking On Manifolds
% Version 0.2
%
% Copyright by Markus Breitenbach and Gregory Z. Grudic
% This code is for your personal and research use only.
%
% http://www.cs.colorado.edu/~grudic/
% http://ucsu.colorado.edu/~breitenm/
%
% This software is provided "as is," without warranty of any kind, express
% or implied.  In no event shall the authors be held liable
% for any direct, indirect, incidental, special or consequential damages
% arising out of the use of or inability to use this software.
%


% Generate some Data

close all

deg = 0:0.05:2*pi;
rad_min = 0.95;
rad_max = 1.0;
X1 = zeros(length(deg),3);
for i=1:length(deg)
    r = rad_min + rand(1,1)*(rad_max-rad_min);
    X1(i,1) = r * cos(deg(i));
    X1(i,2) = r * sin(deg(i));
    X1(i,3) = (i/length(deg));
end


X2 = zeros(length(deg),3);
for i=1:length(deg)
    r = rad_min + rand(1,1)*(rad_max-rad_min);
    X2(i,1) = r * cos(deg(i) + (2 * pi/3));
    X2(i,2) = r * sin(deg(i) + (2 * pi/3));
    X2(i,3) = (i/length(deg));
end

X3 = zeros(length(deg),3);
for i=1:length(deg)
    r = rad_min + rand(1,1)*(rad_max-rad_min);
    X3(i,1) = r * cos(deg(i) + (4 * pi/3));
    X3(i,2) = r * sin(deg(i) + (4 * pi/3));
    X3(i,3) = (i/length(deg));
end

% plot the original data

figure;
plot3(X1(:,1),X1(:,2),X1(:,3),'r+');
hold on
plot3(X2(:,1),X2(:,2),X2(:,3),'g*');
plot3(X3(:,1),X3(:,2),X3(:,3),'bs');
hold off
pause(1); % give matlab a chance to plot

X = [X1;X2;X3];

% Now lets cluster the data

[num_ex,dim] = size(X);

lrn_par = Set_Default_Learning_Paramters;

% distance paramters
lrn_par.DST_TYPE = 1; % 1 for Euclidean, 2 for dot

% Optimization for both ALPHA and SIGMA together
lrn_par.OPT_S_A = 1; % set to 1 to optimize for both alpha and sigma

%lrn_par.my_alpha = 0.99;
%lrn_par.my_sigma= 0.039;
%lrn_par.Cluster_Search = 3;

lrn_par.Cluster_Search = [2,3,4,5]; % This defines the clusters to be evaluated

%Clustering Example
model_cluster = LG_Cluster(X,lrn_par);

%[un_class] = Classify_New_Data(X,model_cluster);

colors=['r+';'go';'b^';'y*';'cs'];
figure
hold on;
for i=1:num_ex
    ii = find(model_cluster.Y(i,:)>0);
    plot3(X(i,1),X(i,2),X(i,3), colors(ii(1),:) );
end;

return;

figure;
ind_1 = model_cluster.Class_Outlier(1).ind;
plot3(X(ind_1,1),X(ind_1,2),X(ind_1,3),'r+');
hold on
ind_2 = model_cluster.Class_Outlier(2).ind;
plot3(X(ind_2,1),X(ind_2,2),X(ind_2,3),'go');
ind_3 = model_cluster.Class_Outlier(3).ind;
plot3(X(ind_3,1),X(ind_3,2),X(ind_3,3),'b^');
xlabel('x');
ylabel('y');
zlabel('z');
hold off
t99 = 0;

t0 = 0;


